/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.Writer;
import java.lang.ref.Reference;
import java.lang.ref.SoftReference;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.WindowConstants;
import javax.swing.tree.TreeNode;

import org.mesopotamia.sql.LoadError;
import org.mesopotamia.sql.MesopotamiaEngine;
import org.w3c.dom.Element;

import biz.hammurapi.convert.ConverterClosure;
import biz.hammurapi.convert.ConvertingService;
import biz.hammurapi.legacy.review.Signed;
import biz.hammurapi.swing.Browser;
import biz.hammurapi.swing.Visualizable;
import biz.hammurapi.util.Attributable;
import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.ConvertingCollection;
import biz.hammurapi.util.VisitableBase;
import biz.hammurapi.util.Visitor;
import biz.hammurapi.xml.dom.DomSerializable;


/**
 * Represents source unit, e.g. java source file
 * @author Pavel Vlasov
 * @revision $Revision: 1.1 $
 */
public class AstSourceUnit extends SourceUnit {
	
	private Collection<Object> languageElements;
	
	/**
	 * Instances of SourceUnit shall be created only by
	 * repository
	 * @throws SQLException 
	 */
	protected AstSourceUnit(
			org.mesopotamia.sql.SourceUnit dbData, 
			final Scan scan, 
			RepositoryLanguage repoLanguage, 
			Collection<Number> loadLevels) throws MesopotamiaException {
		super(dbData, scan, repoLanguage, loadLevels);
		
		SyntaxTree st = getSyntaxTree();
		if (st==null) {
			languageElements = EMPTY_UNMODIFIABLE_LIST;
		} else {
			ConvertingCollection rootLanguageElements = new ConvertingCollection(
										st.getRoots(),
										new ConverterClosure() {
			
											public Object convert(Object source) {
												NodeData mn = (NodeData) source;
												LanguageElementHandle handle = new LanguageElementHandle(mn.getSourceUnitId(), mn.getId(), null, null);
												return scan.getLanguageElement(handle);
											}
											
										},
										null);
			
			languageElements = Collections.unmodifiableCollection(rootLanguageElements);
		}
	}
	
	/**
	 * Helper method for ANTLR-based languages. Returns data for level "ast" cast to SyntaxTree.
	 * @return
	 */
	public SyntaxTree getSyntaxTree() {
		return (SyntaxTree) getLevelData("ast");
	}

	protected void acceptChildren(Visitor visitor) {
		new CollectionVisitable(languageElements, false).accept(visitor);		
	}
	
	/**
	 * @return Root language elements
	 */
	public Collection<Object> getLanguageElements() {
		return languageElements;
	}

	public void toDom(Element holder) {
		super.toDom(holder);
	
		Element le=holder.getOwnerDocument().createElement("language-elements");
		holder.appendChild(le);
		DomSerializable ds = (DomSerializable) ConvertingService.convert(languageElements, DomSerializable.class);
		ds.toDom(le);
		
	}
	
	public Token getToken(int position) {
		return (Token) ((ArrayList<Token>) getLevelData("token")).get(position);
	}
		
}
