/*
  * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.MessageDigest;

/**
 * java.io.File based Source for parsing
 * 
 * @author sourceeater
 * @revision $Revision: 1.2 $
 */
public class FileSource implements Source {

	private final File theFileSource;
	private String path;

	public FileSource(File aFileSource, String path) {
		if (aFileSource==null) {
			throw new NullPointerException();
		}
		if (!aFileSource.exists()) {
			throw new IllegalArgumentException("File does not exist: "+aFileSource.getAbsolutePath());
		}
		if (!aFileSource.isFile()) {
			throw new IllegalArgumentException("Not a file: "+aFileSource.getAbsolutePath());
		}
		this.theFileSource = aFileSource;
		this.path=path.replace(File.separatorChar, '/');
	}

	public long getSize() {
		return theFileSource.length();
	}

	public String getDigest(MessageDigest digest) throws MesopotamiaException {
		try {
			digest.reset();
			FileInputStream fis=new FileInputStream(theFileSource);		
			byte[] buf=new byte[4096];
			int l;
			try {
				while ((l=fis.read(buf))!=-1) {
					digest.update(buf, 0, l);
				}
			} finally {
				fis.close();
			}
			byte[] dba = digest.digest();
	    	StringBuffer dBuf=new StringBuffer();		
		    for (int i=0; i<dba.length; i++) {
		    	String str=Integer.toHexString(dba[i] & 0xFF);
				if (str.length()==1) {
		    		dBuf.append("0");
		    	}
		    	dBuf.append(str);
		    }			
		    return dBuf.toString();
		} catch (IOException e) {
			throw new MesopotamiaException("Could not calculate checksum for file: "+path, e);
		}
	}

	public String getPath() {
		return path;
	}

	public String getName() {
		return theFileSource.getName();
	}

	public Object get() {
		return this.theFileSource;
	}

	public long getLastModified() {
		return theFileSource.lastModified();
	}

	public String toString() {
		return getClass().getName()+": "+path;
	}
}
