/*
  * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * For a given start directory iterates recursivly over directory structure to
 * provide Sources from a filesystem.
 * 
 * @author sourceeater
 * @revision $Revision: 1.2 $
 */
public class FileSourceIterator implements SourceIterator {
	
	private Iterator<FileSource> sourceIterator;

	public FileSourceIterator(Collection<File> rootFiles) {
		Iterator<File> it=rootFiles.iterator();
		while (it.hasNext()) {
			File start=it.next();
			int startPos = start.getAbsolutePath().length()+1;
			Collection<FileSource> sources=new ArrayList<FileSource>();
			enumerateFiles(startPos, start, sources);
			sourceIterator=sources.iterator();
		}
	}

	private static void enumerateFiles(int startPos, File file, Collection<FileSource> receiver) {
		if (file.isFile()) {
			String path=file.getAbsolutePath();
			receiver.add(new FileSource(file, path.length()<startPos ? file.getName() : path.substring(startPos)));
		} else if (file.isDirectory()) {
			File[] fa=file.listFiles();
			for (int i=0; i<fa.length; i++) {
				enumerateFiles(startPos, fa[i], receiver);
			}
		} else {
			// Special file - do nothing.
		}
	}
	
	public Source nextSource() {		
		if (sourceIterator.hasNext()) {
			return sourceIterator.next();
		} 
		
		return null;
	} // nextSource

} // FileSourceIterator
