/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.Wrapper;
import biz.hammurapi.xml.dom.AbstractDomObject;
import biz.hammurapi.xml.dom.DOMUtils;


public class FileSourceIteratorComponent implements Component, Wrapper, DomConfigurable {
	private FileSourceIterator master;
	private Collection<File> sources=new ArrayList<File>();

	public FileSourceIteratorComponent() {
		super();
	}

	public void start() throws ConfigurationException {
		master=new FileSourceIterator(sources);
	}

	public void stop() throws ConfigurationException {
		master=null;
		sources.clear();
	}

	public void setOwner(Object owner) {
		// This component doesnt' care about the environment.
	}

	public Object getMaster() {
		return master;
	}

	public void configure(Node configNode, Context context, ClassLoader classLoader) throws ConfigurationException {
		try {
			NodeList nl=DOMUtils.selectNodeList(configNode, "source");
			for (int i=0, l=nl.getLength(); i<l; ++i) {
				sources.add(new File(AbstractDomObject.getElementText(nl.item(i))));
			}
		} catch (Exception e) {
			throw new ConfigurationException("Cannot configure "+this+": "+e, e);
		}
		
	}

}
