/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.io.Serializable;

import org.w3c.dom.Element;

import biz.hammurapi.xml.dom.DomSerializable;

public class LanguageElementHandle implements Serializable, DomSerializable {
	private Class contextClass;
	private Class targetClass;
	private int id;
	private int sourceUnitId;
		
	private String targetClassName;
	private String contextClassName;	

	/**
	 * @param id
	 * @param contextClass
	 */
	public LanguageElementHandle(int sourceUnitId, int id, Class contextClass, Class targetClass) {
		super();
		this.sourceUnitId = sourceUnitId;
		this.id = id;
		this.contextClass = contextClass;
		if (contextClass!=null) {
			contextClassName=contextClass.getName();
		}
		this.targetClass = targetClass;
		if (targetClass!=null) {
			targetClassName=targetClass.getName();
		}
	}

	public Class getContextClass() {
		return contextClass;
	}

	public Class getTargetClass() {
		return targetClass;
	}

	public int getId() {
		return id;
	}
	
	public int getSourceUnitId() {
		return sourceUnitId;
	}

	public void toDom(Element holder) {
		holder.setAttribute("id", String.valueOf(id));
		holder.setAttribute("source-unit", String.valueOf(sourceUnitId));
		if (contextClass!=null) {
			holder.setAttribute("context", contextClass.getName());
		}
		if (targetClass!=null) {
			holder.setAttribute("target", targetClass.getName());
		}
	}	
	
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((contextClassName == null) ? 0 : contextClassName.hashCode());
		result = PRIME * result + id;
		result = PRIME * result + sourceUnitId;
		result = PRIME * result + ((targetClassName == null) ? 0 : targetClassName.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final LanguageElementHandle other = (LanguageElementHandle) obj;
		if (id != other.id) {
			return false;
		}
		if (sourceUnitId != other.sourceUnitId) {
			return false;
		}
		
		if (contextClassName == null) {
			if (other.contextClassName != null) {
				return false;
			}
		} else if (!contextClassName.equals(other.contextClassName)) {
			return false;
		}
		
		if (targetClassName == null) {
			if (other.targetClassName != null) {
				return false;
			}
		} else if (!targetClassName.equals(other.targetClassName)) {
			return false;
		}
		
		return true;
	}

	public String toString() {
		return "[Language element handle] "+sourceUnitId+"/"+id+" ("+contextClass+") -> "+targetClass;
	}
}