/*
  * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import gnu.trove.TIntHashSet;
import gnu.trove.TIntIterator;

import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Logger;

import org.mesopotamia.sql.LoaderImpl;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.StringConfigurable;
import biz.hammurapi.eval.CircularReferenceException;
import biz.hammurapi.sql.RowProcessor;
import biz.hammurapi.sql.SQLExceptionEx;


/**
 * Holds loader instance
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class LoaderEntry extends LoaderImpl {
	private Loader loader;
	private TIntHashSet masters=new TIntHashSet();
	private RepositoryLanguage repoLanguage;
	private static final Logger logger = Logger.getLogger(LoaderEntry.class.getName());

	/**
	 * @param rs
	 * @throws SQLException
	 */
	public LoaderEntry(RepositoryLanguage repoLanguage, ResultSet rs) throws SQLException {
		super(rs);
		this.repoLanguage=repoLanguage;
		try {
			loader=(Loader) repoLanguage.getClassLoader().loadClass(getLoaderClass()).getConstructor(
					new Class[] {RepositoryLanguage.class, LoaderEntry.class}).
						newInstance(new Object[] {repoLanguage, this});
			
			if (getLoaderConfig()!=null) {
				if (loader instanceof StringConfigurable) {
					try {
						((StringConfigurable) loader).configure(getLoaderConfig(), null);
					} catch (ConfigurationException e) {
						throw new SQLExceptionEx("Could not configure loader: "+e, e);
					}
				} else {
					logger.warning("Loader "+loader.getClass().getName()+
							" has not null configuration, but it doesn't implement "+
							StringConfigurable.class.getName());
				}
			}
			
			repoLanguage.getFactory().getEngine().processLoaderDependencyByDependentLevel(
					rs.getInt("ID"),
					new RowProcessor() {

						public boolean process(ResultSet rs) throws SQLException {
							masters.add(rs.getInt("DEPENDS_ON_LEVEL_ID"));
							return true;
						}
						
					});
		} catch (InstantiationException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		} catch (IllegalAccessException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		} catch (ClassNotFoundException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		} catch (SecurityException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		} catch (InvocationTargetException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		} catch (NoSuchMethodException e) {
			throw new SQLExceptionEx("Cannot instantiate loader "+getLoaderClass(), e);
		}
	}

	Loader getLoader() {
		return loader;
	}
	
	boolean dependsOn(int master, int originalMaster) {
		if (getId()==originalMaster) {
			throw new CircularReferenceException("Circular reference in loader dependency: ["+originalMaster+"] "+getLevel());
		}
		
		if (masters.contains(master)) {
			return true;
		}
		
		TIntIterator it=masters.iterator();
		while (it.hasNext()) {
			if (((LoaderEntry) repoLanguage.loadersMap.get(it.next())).dependsOn(master, originalMaster)) {
				return true;
			}
		}
		
		return false;
	}
	
	boolean isScanDependent() {
		return isScanDependent(null);
	}
	
	/**
	 * @return true if this loader is not dependent of any others
	 */
	boolean isIndependent() {
		return masters.isEmpty();
	}
	
	private boolean isScanDependent(String startingLevel) {
		if (getScanDependent()) {
			return true;
		}
		
		if (getLevel().equals(startingLevel)) {
			throw new CircularReferenceException("Circular reference in loader dependency: "+getLevel());
		}
		
		String actualStartingLevel = startingLevel==null ? getLevel() : startingLevel;
		TIntIterator mit=masters.iterator();
		while (mit.hasNext()) {
			if (((LoaderEntry) repoLanguage.loadersMap.get(mit.next())).isScanDependent(actualStartingLevel)) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * @param loadLevels
	 * @return true if all prerequisite levels were loaded
	 * and this level wasn't.
	 */
	boolean isCompatible(TIntHashSet loadLevels) {
		return !loadLevels.contains(getId()) && loadLevels.containsAll(masters.toArray());
	}
	
}