CREATE SCHEMA MESOPOTAMIA AUTHORIZATION DBA;

SET SCHEMA MESOPOTAMIA;

CREATE USER MESOPOTAMIA PASSWORD "" ADMIN;

CREATE CACHED TABLE ERROR_MESSAGE ( 
	ID int identity  NOT NULL PRIMARY KEY,
	ERROR_TYPE varchar(500),
	MESSAGE_TEXT varchar(500) NOT NULL,
	SOURCE_UNIT_ID int,
	SCAN_ID int
);

CREATE CACHED TABLE GLOBAL_PARAMETER ( 
	NAME varchar(50) NOT NULL PRIMARY KEY,
	CLASS_NAME varchar(250) NOT NULL,
	PARAMETER_VALUE varchar(250)
);

CREATE CACHED TABLE LANGUAGE ( 
	NAME varchar(20) NOT NULL,
	VERSION varchar(10) DEFAULT '' NOT NULL,
	DESCRIPTION varchar(2000),
	NAMESPACE_IMPLEMENTATION varchar(500),

	CONSTRAINT PK_LANGUAGE PRIMARY KEY (NAME, VERSION) 
);

CREATE CACHED TABLE LANGUAGE_ELEMENT_CLASS ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	TOKEN_TYPE_ID int NOT NULL,
	CLASS_NAME varchar(500) NOT NULL,
	DESCRIPTION varchar(200),
	CONTEXT_CLASS VARCHAR(500),
	REQUIRES_ENVIRONMENT BIT DEFAULT 0 NOT NULL 
);

CREATE CACHED TABLE LANGUAGE_ELEMENT_REQUIRED_LEVEL ( 
	LEVEL_ID int NOT NULL,
	TOKEN_TYPE_ID int NOT NULL,
	DESCRIPTION varchar(500),
	CONSTRAINT PK_LANGUAGE_ELEMENT_REQUIRED_LEVEL PRIMARY KEY (LEVEL_ID, TOKEN_TYPE_ID) 
);

CREATE CACHED TABLE LOAD_LEVEL ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	LEVEL_ID int NOT NULL,
	SOURCE_UNIT_ID int NOT NULL,
	SCAN_ID int, -- Scan can be null for load levels which are scan-independent (e.g. tokens)
	MESSAGE_ID int, -- Error message id
	LOAD_FAILED BIT DEFAULT '0' NOT NULL,
	LOADER_DATA VARBINARY -- Data provided by the loader. E.g. for content loader it would be simply text, for AST loader it might be XML tree.
);

CREATE UNIQUE INDEX UIX_LOAD_LEVEL_LEVEL_SU_SCAN ON LOAD_LEVEL (LEVEL_ID, SOURCE_UNIT_ID, SCAN_ID);

CREATE CACHED TABLE LOADER ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	LANGUAGE varchar(20) NOT NULL,
	LANGUAGE_VERSION varchar(10) DEFAULT '' NOT NULL,
	LEVEL varchar(10) NOT NULL,
	LOADER_CLASS varchar(500) NOT NULL,
	DESCRIPTION varchar(50),
	REQUIRES_SOURCE BIT DEFAULT '0' NOT NULL,
	REQUIRES_ENVIRONMENT BIT DEFAULT '0' NOT NULL,
	SCAN_DEPENDENT BIT DEFAULT 0 NOT NULL, 
	LOADER_CONFIG VARCHAR(2000)
);

CREATE UNIQUE INDEX UIX_LOADER_SQLC$E_LANGUAGE_LEVEL ON LOADER (LANGUAGE, LANGUAGE_VERSION, LEVEL);

CREATE CACHED TABLE LOADER_DEPENDENCY ( 
	DEPENDENT_LEVEL_ID int NOT NULL,
	DEPENDS_ON_LEVEL_ID int NOT NULL,
	DESCRIPTION varchar(500),
	CONSTRAINT PK_LOADER_DEPENDENCY PRIMARY KEY (DEPENDENT_LEVEL_ID, DEPENDS_ON_LEVEL_ID) 
);

CREATE CACHED TABLE NAMESPACE ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	REPOSITORY int NOT NULL,
	NAME varchar(500) NOT NULL
);

CREATE CACHED TABLE REPOSITORY ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	NAME varchar(50) NOT NULL,
	DESCRIPTION varchar(500)
);

CREATE CACHED TABLE SCAN ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	REPOSITORY int NOT NULL,
	SCAN_DATE timestamp NOT NULL,
	DESCRIPTION varchar(200)
);

CREATE CACHED TABLE SOURCE_UNIT ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	NAME varchar(100) NOT NULL,
	PATH varchar(500) NOT NULL,
	UNIT_SIZE bigint,
	UNIT_DIGEST varchar(50),
	LAST_MODIFIED datetime, 
	DIGEST_ALGORITHM varchar(30),
	NAMESPACE_ID int,
	PREV_REVISION int,
	LANGUAGE varchar(20) NOT NULL,
	LANGUAGE_VERSION varchar(10) NOT NULL,
	REPOSITORY_ID int NOT NULL
);

CREATE CACHED TABLE SOURCE_UNIT_CLASS ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	LANGUAGE varchar(20) NOT NULL,
	LANGUAGE_VERSION varchar(10) DEFAULT '' NOT NULL,
	CLASS_NAME varchar(500) NOT NULL,
	DESCRIPTION varchar(200)
);

CREATE CACHED TABLE SOURCE_UNIT_REQUIRED_LEVEL ( 
	LEVEL_ID int NOT NULL,
	SOURCE_UNIT_CLASS_ID int NOT NULL,
	DESCRIPTION varchar(500),

	CONSTRAINT PK_SOURCE_UNIT_REQUIRED_LEVEL PRIMARY KEY (SOURCE_UNIT_CLASS_ID, LEVEL_ID)
);

CREATE CACHED TABLE SOURCE_UNIT_SCAN ( 
	SCAN_ID int NOT NULL,
	SOURCE_UNIT_ID int NOT NULL,

	CONSTRAINT PK_SOURCE_UNIT_SCAN PRIMARY KEY (SCAN_ID, SOURCE_UNIT_ID)
);

CREATE CACHED TABLE TOKEN_TYPE ( 
	ID int IDENTITY  NOT NULL PRIMARY KEY,
	LANGUAGE varchar(20) NOT NULL,
	LANGUAGE_VERSION varchar(10) DEFAULT '' NOT NULL,
	TOKEN_TYPE int NOT NULL,
	TOKEN_NAME varchar(50) NOT NULL,
	DESCRIPTION varchar(500),
	IS_EOF BIT DEFAULT '0' NOT NULL,
	IS_WHITESPACE BIT DEFAULT '0' NOT NULL,
	RENDER_STYLE varchar(50)
);

ALTER TABLE ERROR_MESSAGE ADD CONSTRAINT FK_ERROR_MESSAGE_SQLC$SCAN 
	FOREIGN KEY (SCAN_ID) REFERENCES SCAN (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE ERROR_MESSAGE ADD CONSTRAINT FK_ERROR_MESSAGE_SQLC$SOURCE_UNIT 
	FOREIGN KEY (SOURCE_UNIT_ID) REFERENCES SOURCE_UNIT (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LANGUAGE_ELEMENT_CLASS ADD CONSTRAINT FK_LANGUAGE_ELEMENT_CLASS_SQLC$TOKEN_TYPE 
	FOREIGN KEY (TOKEN_TYPE_ID) REFERENCES TOKEN_TYPE (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LANGUAGE_ELEMENT_REQUIRED_LEVEL ADD CONSTRAINT FK_LANGUAGE_ELEMENT_REQUIRED_LEVEL_SQLC$LEVEL 
	FOREIGN KEY (LEVEL_ID) REFERENCES LOADER (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOAD_LEVEL ADD CONSTRAINT FK_LOAD_LEVEL_SQLC$SCAN 
	FOREIGN KEY (SCAN_ID) REFERENCES SCAN (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOAD_LEVEL ADD CONSTRAINT FK_LOAD_LEVEL_SQLC$ERROR_MESSAGE 
	FOREIGN KEY (MESSAGE_ID) REFERENCES ERROR_MESSAGE (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOAD_LEVEL ADD CONSTRAINT FK_LOAD_LEVEL_SQLC$LEVEL 
	FOREIGN KEY (LEVEL_ID) REFERENCES LOADER (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOAD_LEVEL ADD CONSTRAINT FK_LOAD_LEVEL_SQLC$SOURCE_UNIT 
	FOREIGN KEY (SOURCE_UNIT_ID) REFERENCES SOURCE_UNIT (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOADER ADD CONSTRAINT FK_LOADER_SQLC$LANGUAGE 
	FOREIGN KEY (LANGUAGE, LANGUAGE_VERSION) REFERENCES LANGUAGE (NAME, VERSION)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOADER_DEPENDENCY ADD CONSTRAINT FK_LOADER_DEPENDENCY_SQLC$DEPENDENT_LEVEL 
	FOREIGN KEY (DEPENDENT_LEVEL_ID) REFERENCES LOADER (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE LOADER_DEPENDENCY ADD CONSTRAINT FK_LOADER_DEPENDENCY_SQLC$DEPENDS_ON_LEVEL 
	FOREIGN KEY (DEPENDS_ON_LEVEL_ID) REFERENCES LOADER (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE NAMESPACE ADD CONSTRAINT FK_NAMESPACE_SQLC$REPOSITORY 
	FOREIGN KEY (REPOSITORY) REFERENCES REPOSITORY (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SCAN ADD CONSTRAINT FK_SCAN_SQLC$REPOSITORY 
	FOREIGN KEY (REPOSITORY) REFERENCES REPOSITORY (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT ADD CONSTRAINT FK_SOURCE_UNIT_SQLC$PREV_REVISION 
	FOREIGN KEY (PREV_REVISION) REFERENCES SOURCE_UNIT (ID);

ALTER TABLE SOURCE_UNIT ADD CONSTRAINT FK_SOURCE_UNIT_SQLC$NAMESPACE 
	FOREIGN KEY (NAMESPACE_ID) REFERENCES NAMESPACE (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT_CLASS ADD CONSTRAINT FK_SOURCE_UNIT_CLASS_SQLC$LANGUAGE 
	FOREIGN KEY (LANGUAGE, LANGUAGE_VERSION) REFERENCES LANGUAGE (NAME, VERSION)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT_REQUIRED_LEVEL ADD CONSTRAINT FK_SOURCE_UNIT_REQUIRED_LEVEL_SQLC$LOADER 
	FOREIGN KEY (LEVEL_ID) REFERENCES LOADER (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT_REQUIRED_LEVEL ADD CONSTRAINT FK_SOURCE_UNIT_REQUIRED_LEVEL_SQLC$CLASS 
	FOREIGN KEY (SOURCE_UNIT_CLASS_ID) REFERENCES SOURCE_UNIT_CLASS (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT_SCAN ADD CONSTRAINT FK_SOURCE_UNIT_SCAN_SQLC$SCAN 
	FOREIGN KEY (SCAN_ID) REFERENCES SCAN (ID);

ALTER TABLE SOURCE_UNIT_SCAN ADD CONSTRAINT FK_SOURCE_UNIT_SCAN_SQLC$SOURCE_UNIT 
	FOREIGN KEY (SOURCE_UNIT_ID) REFERENCES SOURCE_UNIT (ID)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE TOKEN_TYPE ADD CONSTRAINT FK_TOKEN_TYPE_SQLC$LANGUAGE 
	FOREIGN KEY (LANGUAGE, LANGUAGE_VERSION) REFERENCES LANGUAGE (NAME, VERSION)
ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE SOURCE_UNIT ADD CONSTRAINT FK_SOURCE_UNIT_SQLC$LANGUAGE 
	FOREIGN KEY (LANGUAGE, LANGUAGE_VERSION) REFERENCES LANGUAGE (NAME, VERSION);

ALTER TABLE SOURCE_UNIT ADD CONSTRAINT FK_SOURCE_UNIT_SQLC$REPOSITORY 
	FOREIGN KEY (REPOSITORY_ID) REFERENCES REPOSITORY (ID)
ON DELETE CASCADE ON UPDATE CASCADE;