package org.mesopotamia;

import java.util.HashMap;
import java.util.Map;

import biz.hammurapi.util.VisitableBase;

/**
 * Contains only data and no references to other objects.
 * @author Pavel Vlasov
 *
 */
public class NodeData extends VisitableBase implements Cloneable {
	
//	private StackTraceElement[] sf = new Exception().getStackTrace(); // For testing

	private int column;
	private int line;
	private String text;
	private int type;
	private int leftColumn;
	private int id;
	private Integer firstToken;
	private Integer lastToken;
	private Integer token;
	private int sourceUnitId;
	private int position;
	private int sameTypeIndex;
	
	private Map references = new HashMap();
	
	public NodeData() {
		// default constructor
	}
	
	@Override
	public Object clone() throws CloneNotSupportedException {
		NodeData ret = (NodeData) super.clone();
		ret.setColumn(getColumn());
		ret.setLine(getLine());
		ret.setText(getText());
		ret.setType(getType());
		ret.setLeftColumn(getLeftColumn());
		ret.setId(getId());
		ret.setFirstToken(getFirstToken());
		ret.setLastToken(getLastToken());
		ret.setToken(getToken());
		ret.setSourceUnitId(getSourceUnitId());
		ret.setPosition(getPosition());
		ret.setSameTypeIndex(getSameTypeIndex());
		
		return ret;
	}
	

	public int getSourceUnitId() {
		return sourceUnitId;
	}

	public void setSourceUnitId(int sourceUnitId) {
		this.sourceUnitId = sourceUnitId;
	}

	public int getColumn() {
		return column;
	}

	public void setColumn(int column) {
		this.column = column;
	}

	public Integer getFirstToken() {
		return firstToken;
	}

	public void setFirstToken(Integer firstToken) {
		this.firstToken = firstToken;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Integer getLastToken() {
		return lastToken;
	}

	public void setLastToken(Integer lastToken) {
		this.lastToken = lastToken;
	}

	public int getLeftColumn() {
		return leftColumn;
	}

	public void setLeftColumn(int leftColumn) {
		this.leftColumn = leftColumn;
	}

	public int getLine() {
		return line;
	}

	public void setLine(int line) {
		this.line = line;
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public Integer getToken() {
		return token;
	}

	public void setToken(Integer token) {
		this.token = token;
	}

	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}
	
	public int getPosition() {
		return position;
	}
	
	public void setPosition(int position) {
		this.position = position;
	}

	public void setSameTypeIndex(int sameTypeIndex) {
		this.sameTypeIndex = sameTypeIndex;
	}
	
	public int getSameTypeIndex() {
		return sameTypeIndex;
	}
	
	public String toString() {
		return "["+getClass().getName()+"] "+sourceUnitId+"/"+id+" "+line+":"+column;
	}
	
	/**
	 * Map of reference name to reference value, e.g. type -&gt; 132.
	 * @return
	 */
	public Map getReferences() {
		return references;
	}
}
