/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.mesopotamia.sql.MesopotamiaEngine;
import org.mesopotamia.sql.ScanImpl;

import biz.hammurapi.sql.IdentityGenerator;
import biz.hammurapi.sql.IdentityRetriever;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.Transaction;

/**
 * @author Pavel Vlasov
 * @revision $Revision: 1.2 $
 */
public class Repository {
	private RepositoryFactory factory;
	private org.mesopotamia.sql.Repository dbData;
	
	/**
	 * Repository is instantiated only by RepositoryFactory
	 * @param factory
	 * @param language
	 * @param languageVersion
	 * @param name
	 */
	Repository(RepositoryFactory factory, org.mesopotamia.sql.Repository dbData) {
		this.factory=factory;
		this.dbData=dbData;
	}
	
	public RepositoryFactory getFactory() {
		return factory;
	}

	public String getDescription() {
		return dbData.getDescription();
	}

	public int getId() {
		return dbData.getId();
	}

	public String getName() {
		return dbData.getName();
	}
	
	public Collection<Scan> getScans() throws MesopotamiaException {
		Collection<Scan> ret = new ArrayList<Scan>();
		Iterator<org.mesopotamia.sql.Scan> it = factory.getEngine().getScanByRepository(getId()).iterator();
		while (it.hasNext()) {
			org.mesopotamia.sql.Scan sd = it.next();
			ret.add(factory.getScan(sd.getId()));
		}
		return ret;
	}
	
	public Scan createScan(
			SourceIterator sourceIterator, 
			LanguageSelector selector, 
			String description, 
			String checkSumName, 
			Object environment, 
			ScanLoadListener listener) throws MesopotamiaException {
		try {
			final ScanImpl si=new ScanImpl(true);
			si.setDescription(description);
			si.setScanDate(new Timestamp(System.currentTimeMillis()));
			si.setRepository(dbData.getId());
			factory.getProcessor().executeTransaction(new Transaction() {
	
				public boolean execute(SQLProcessor processor) throws SQLException {					
					if (factory.getIdentityManager() instanceof IdentityGenerator) {
						si.setId(((IdentityGenerator) factory.getIdentityManager()).generate(processor.getConnection(), "REPOSITORY"));
					}
					new MesopotamiaEngine(processor).insertScan(si);
					if (factory.getIdentityManager() instanceof IdentityRetriever) {
						si.setId(((IdentityRetriever) factory.getIdentityManager()).retrieve(processor.getConnection()));
					}
					return true;
				}
				
			});
			
			Scan scan = new Scan(this, si, sourceIterator, selector, checkSumName, environment, listener);
			factory.addScan(scan);
			return scan;
		} catch (SQLException e) {
			throw new MesopotamiaException("Cannot create repository", e);
		}
	}
	
	/**
	 * Removes all scans and repository from the database.
	 * @throws MesopotamiaException 
	 */
	public void delete() throws MesopotamiaException {
		try {
			MesopotamiaEngine engine = factory.getEngine();
			engine.deleteSourceUnitByRepository(getId()); // To avoid potential cascading conflict with Scan. Cascades to source_unit_scan
			engine.deleteRepository(dbData.getId()); // Cascades to scan
		} catch (SQLException e) {
			throw new MesopotamiaException("Cannot delete repository", e);
		}
	}
}
