/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia;

import java.sql.SQLException;
import java.util.Timer;

import javax.sql.DataSource;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import biz.hammurapi.config.Component;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.config.Context;
import biz.hammurapi.config.DomConfigurable;
import biz.hammurapi.config.Wrapper;
import biz.hammurapi.persistence.StringStorage;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.util.Worker;


public class RepositoryFactoryComponent implements Component, Wrapper, DomConfigurable {

	private RepositoryFactory master;
	private Object owner;
	private String dataSourceName="/datasource";
	private String timerName="/timer";
	private String stringStorageName="/string-storage";
	private String workerName;

	public RepositoryFactoryComponent() {
		super();
	}

	public void start() throws ConfigurationException {
		try {
			Context context = (Context) owner;
			DataSource dataSource = (DataSource) context.get(dataSourceName);
			master=new RepositoryFactory(
					new SQLProcessor(dataSource, null), 
					workerName==null ? null : (Worker) context.get(workerName), 
					(Timer) context.get(timerName),
					(StringStorage) context.get(stringStorageName),
					getClass().getClassLoader()); 
		} catch (SQLException e) {
			throw new ConfigurationException("Cannot create repository factory: "+e,e);
		}
	}

	public void stop() throws ConfigurationException {
		// Nothing to do
	}

	public void setOwner(Object owner) {
		this.owner=owner;
	}

	public Object getMaster() {
		return master;
	}

	public void configure(Node configNode, Context context, ClassLoader classLoader) throws ConfigurationException {
		Element ce=(Element) configNode;
		if (ce.hasAttribute("data-source")) {
			dataSourceName=ce.getAttribute("data-source");			
		}
		
		if (ce.hasAttribute("worker")) {
			workerName=ce.getAttribute("worker");
		}
	}

}
