package org.mesopotamia;

import gnu.trove.TIntObjectHashMap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import biz.hammurapi.util.Attributable;
import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.VisitableBase;
import biz.hammurapi.util.Visitor;

/**
 * Raw source unit
 * @author Pavel Vlasov
 *
 */
public class SyntaxTree extends VisitableBase implements Attributable {
	private List<MesopotamiaNode> roots = new ArrayList<MesopotamiaNode>();
	private TIntObjectHashMap nodeMap = new TIntObjectHashMap();
	
	public List<MesopotamiaNode> getRoots() {
		return roots;
	}
	
	public MesopotamiaNode findNode(int id) {
		return (MesopotamiaNode) nodeMap.get(id);
	}
	
	public void mapNode(MesopotamiaNode node) {
		nodeMap.put(node.getId(), node);
	}
	
	protected void acceptChildren(Visitor visitor) {
		new CollectionVisitable(roots, false).accept(visitor);
	}
	
	private Map<Object, Object> attributes = new HashMap<Object, Object>();

	public Object getAttribute(Object key) {
		return attributes.get(key);
	}

	public Object removeAttribute(Object key) {
		return attributes.remove(key);
	}

	public void setAttribute(Object key, Object value) {
		attributes.put(key, value);		
	}	
}
