/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */

package org.mesopotamia;

import org.mesopotamia.sql.TokenType;
import org.w3c.dom.Element;

import biz.hammurapi.xml.dom.DomSerializable;

/**
 * Token
 * @author Pavel Vlasov
 *
 */
public class Token implements DomSerializable {

	org.mesopotamia.sql.TokenType type;
	int col;
	int line;
	int position;
	String text;
	private SourceUnit su;

	/**
	 * Can be created only by Source unit.
	 * @param type
	 * @param position
	 * @param line
	 * @param col
	 * @param text
	 */
	Token(SourceUnit su, TokenType type, int position, int line, int col, String text) {
		super();
		this.type = type;
		this.position = position;
		this.line = line;
		this.col = col;
		this.text = text;
		this.su = su;
	}
	
	

	public void toDom(Element holder) {
		setAttribute(holder, "column", String.valueOf(col));
		setAttribute(holder, "position", String.valueOf(position));
		setAttribute(holder, "type", String.valueOf(type.getId()));
		//type.toDom((Element) holder.appendChild(holder.getOwnerDocument().createElement("type")));
		
		if (text!=null) {
			holder.appendChild(holder.getOwnerDocument().createTextNode(text));
		}
	}
	
	private void setAttribute(Element holder, String name, String value) {
		if (value!=null) {
			holder.setAttribute(name, value);
		}
	}
	
	public String toString() {
		return "["+getClass().getName()+"] "+getTokenName()+" "+getLine()+":"+getCol()+" "+getText();
	}
	
	public SourceUnit getSourceUnit() {
		return su;
	}

	public String getDescription() {
		return type.getDescription();
	}

	public int getTypeId() {
		return type.getId();
	}

	public boolean getIsEof() {
		return type.getIsEof();
	}

	public boolean getIsWhitespace() {
		return type.getIsWhitespace();
	}

	public String getLanguage() {
		return type.getLanguage();
	}

	public String getLanguageVersion() {
		return type.getLanguageVersion();
	}

	public String getRenderStyle() {
		return type.getRenderStyle();
	}

	public String getTokenName() {
		return type.getTokenName();
	}

	public int getTokenType() {
		return type.getTokenType();
	}

	public int getCol() {
		return col;
	}

	public int getLine() {
		return line;
	}

	public int getPosition() {
		return position;
	}

	public String getText() {
		return text;
	}	
}
