/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia.util;

import java.util.Collection;
import java.util.Iterator;

import org.mesopotamia.MesopotamiaRuntimeException;

import antlr.collections.AST;

/**
 * Parses attribute definition.
 * 
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class Attribute {
	private boolean isList;
	private boolean isString;
	private String name;
	private String type;
	private String path;
	private String packageName;
	private RuleDefinition owner;
	
	public Attribute(RuleDefinition owner, AST node, String packageName) {
		this.packageName=packageName;
		this.owner=owner;
		StringBuffer sb=new StringBuffer();
		for (AST child=node.getFirstChild(); child!=null; child=child.getNextSibling()) {
			switch (child.getType()) {
			case BnfTokenTypes.LITERAL_attribute:
				isList=false;
				isString=false;
				break;
			case BnfTokenTypes.LITERAL_list:
				isList=true;
				break;
			case BnfTokenTypes.LITERAL_string:
				isString=true;
				break;
			case BnfTokenTypes.IDENT:
			case BnfTokenTypes.DOT:
				StringBuffer nameBuf=new StringBuffer();
				typeName(child, nameBuf);
				if (name==null) {
					name=nameBuf.toString();
					type=name;
				} else {
					type=nameBuf.toString();
				}
				break;
			case BnfTokenTypes.PATH:
				if (sb.length()>0) {
					sb.append("|");
				}
				StringBuffer peb=new StringBuffer();
				for (AST pe=child.getFirstChild(); pe!=null; pe=pe.getNextSibling()) {
					if (peb.length()>0) {
						peb.append("/");
					}
					AST typeNode = pe.getFirstChild();
					switch (typeNode.getType()) {
						case BnfTokenTypes.TYPE_REF:
							peb.append("#");
							if (typeNode.getFirstChild().getType()==BnfTokenTypes.IDENT) {
								peb.append(packageName);
								peb.append(".");
							}
							typeName(typeNode.getFirstChild(), peb);
							break;
						case BnfTokenTypes.EXCL:
							owner.getModel().checkTokenName(typeNode.getFirstChild().getText());
							peb.append("!");
							peb.append(typeNode.getFirstChild().getText());
							break;							
						case BnfTokenTypes.IDENT:
							owner.getModel().checkTokenName(typeNode.getText());
							peb.append(typeNode.getText());
							break;
						case BnfTokenTypes.STAR:
						case BnfTokenTypes.DOUBLEDOT:
							peb.append(typeNode.getText());
							break;
						default:
							throw new MesopotamiaRuntimeException("Unexpected token type: "+BnfRecognizer._tokenNames[typeNode.getType()]);
					}
					
					AST indexNode = typeNode.getNextSibling();
					if (indexNode!=null) {
						peb.append("[");
						peb.append(indexNode.getText());
						peb.append("]");
					}
				}
				sb.append(peb);
				break;
			default:
				throw new IllegalArgumentException("Unexpected node type: "+BnfRecognizer._tokenNames[child.getType()]);
			}
		}
		if (sb.length()!=0) {
			path=sb.toString();
		}
	}
	
	static void typeName(AST node, StringBuffer sb) {
		if (node.getType()==BnfTokenTypes.DOT) {
			typeName(node.getFirstChild(), sb);
			sb.append(".");
			typeName(node.getFirstChild().getNextSibling(), sb);
		} else {
			sb.append(node.getText());
		}
	}	
	
	public String getName() {
		return name;
	}
	
	public boolean isList() {
		return isList;
	}
	
	public boolean isString() {
		return isString;
	}
	
	public String getPath() {
		if (path!=null) {
			return path;
		}
		
		// TODO - take match paths from type
		RuleDefinition rule=owner.getModel().rulesMap.get(type);
		if (rule!=null) {
			Collection<Object> matchPaths=rule.getMatchPaths();
			if (!matchPaths.isEmpty()) {
				StringBuffer rsb=new StringBuffer();
				Iterator<Object> it=matchPaths.iterator();
				while (it.hasNext()) {
					MatchPath path=(MatchPath) it.next();
					if (rsb.length()!=0) {
						rsb.append("|");
					}
					if (path.isNegation()) {
						rsb.append("!");
					}					
					rsb.append(path.getTokenType());
				}
				return rsb.toString();
			}
		}
		
		if (type.indexOf('.')==-1) {
			return "#"+packageName+"."+type;
		}
		
		return "#"+type;
	}
	
	public String getType() {
		return type;
	}
}
