/*
 * mesopotamia @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia.util;

import antlr.collections.AST;

public class MatchPath {
	RuleDefinition owner;
	String tokenType;
	String contextType;
	boolean isNegation;
	
	/**
	 * @param owner
	 * @param node
	 */
	public MatchPath(RuleDefinition owner, AST node) {
		super();
		this.owner = owner;
		for (AST child=node.getFirstChild(); child!=null; child=child.getNextSibling()) {
			switch (child.getType()) {
			case BnfTokenTypes.IDENT:
				tokenType=child.getText();
				owner.getModel().checkTokenName(tokenType);
				break;
			case BnfTokenTypes.EXCL:
				isNegation=true;
				tokenType=child.getFirstChild().getText();
				owner.getModel().checkTokenName(tokenType);
				break;
			case BnfTokenTypes.CONTEXT:
				StringBuffer csb=new StringBuffer();
				Attribute.typeName(child.getFirstChild(), csb);
				contextType=csb.toString();
				if (contextType.indexOf('.')==-1) {
					contextType=owner.getModel().getPackage()+"."+contextType;
				}
				break;
			default:
				throw new IllegalStateException("Unexpected node: "+BnfRecognizer._tokenNames[child.getType()]);
			}
		}
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof MatchPath) {
			MatchPath omp = (MatchPath) obj;
			return 
				tokenType.equals(omp.tokenType)
				&& isNegation==omp.isNegation
				&& (contextType==null ? omp.contextType==null : contextType.equals(omp.contextType));
		}
		return false;
	}
	
	public int hashCode() {
		return tokenType.hashCode() ^ (contextType==null ? 0 : contextType.hashCode()) ^ (isNegation ? 0 : 1); 
	}
		
	public String getContextType() {
		return contextType;
	}
	
	public String getTokenType() {
		return tokenType;
	}
	
	public RuleDefinition getOwner() {
		return owner;
	}
	
	public boolean isNegation() {
		return isNegation;
	}
	
	@Override
	public String toString() {
		return getClass().getName()+(isNegation? " [!" : " [")+tokenType+"@"+contextType+"]";
	}
}
